import json
import os
from io import BytesIO

from envs import env

from bunnyhop import base


class Storage(base.BaseBunny):

    def create(self, name, main_storage_region: str = None, replica_regions: list = None):
        api_data = {
            'Name': name,
            'Region': main_storage_region,
            'ReplicationRegions': replica_regions
        }

        response = self.call_api(f"/storagezone", "POST", json_data=api_data)
        if response.get('Id', None):
            return StorageZone(response.get('Password'), **response)
        return response

    def all(self):
        return [StorageZone(i.get('Password'), **i) for i in
                self.call_api(f"/storagezone", "GET")]

    def delete(self, id):
        return self.call_api(f"/storagezone/{id}", "DELETE")

    def get(self, id):
        response = self.call_api(f"/storagezone/{id}", "GET")
        if response.get('Id', None):
            return StorageZone(response.get('Password'), **response)
        return response


class StorageZone(base.BaseStorageBunny):
    Id = base.IntegerProperty()
    UserId = base.CharProperty()
    Name = base.CharProperty()
    Password = base.CharProperty()
    DateModified = base.DateTimeProperty(required=False)
    Deleted = base.BooleanProperty(default_value=False)
    StorageUsed = base.IntegerProperty(required=False)
    FilesStored = base.IntegerProperty(required=False)
    Region = base.CharProperty()
    ReplicationRegions = base.ListProperty()
    PullZones = base.ListProperty()
    ReadOnlyPassword = base.CharProperty(required=False)

    def __str__(self):
        return self.Name

    def delete(self):
        return self.call_api(f"/storagezone/{self.Id}", "DELETE")

    def all(self, folder=''):
        if not folder.endswith('/'):
            folder = f"{folder}/"
        return [StorageObject(self.api_key, self, **i) for i in self.call_storage_api(f"/{self.Name}/{folder}", "GET")]

    def get(self, file_path):
        return self.call_storage_api(f"/{self.Name}/{file_path}", "GET")

    def head_file(self, file_path):
        return self.call_storage_api(f"/{self.Name}/{file_path}", "HEAD")

    def upload_file(self, dest_path, file_name, local_path):
        return self.call_storage_api(f"/{self.Name}/{dest_path}/{file_name}", "PUT",
                                     data=open(local_path, 'rb').read())

    def create_file(self, file_name, content):
        pass

    def create_json(self, key, data_dict):
        f = BytesIO(json.dumps(data_dict).encode())
        return self.call_storage_api(f"/{self.Name}/{key}", "PUT",
                                     data=f.read())


class StorageObject(base.BaseStorageBunny):
    endpoint_url = env('BUNNYCDN_STORAGE_API_ENDPOINT', 'https://storage.bunnycdn.com')

    Guid = base.SlugProperty(required=True)
    StorageZoneName = base.CharProperty(required=True)
    Path = base.CharProperty(required=True)
    ObjectName = base.CharProperty(required=True)
    Length = base.IntegerProperty()
    LastChanged = base.DateTimeProperty()
    ServerId = base.IntegerProperty()
    IsDirectory = base.BooleanProperty()
    UserId = base.SlugProperty(required=True)
    ContentType = base.CharProperty()
    DateCreated = base.DateTimeProperty()
    StorageZoneId = base.IntegerProperty(required=True)
    Checksum = base.CharProperty()
    ReplicatedZones = base.ListProperty()

    def __init__(self,
                 api_key,
                 storage_zone,
                 endpoint_url=None,
                 **kwargs
                 ):
        self.storage_zone = storage_zone
        super().__init__(api_key, endpoint_url=endpoint_url, **kwargs)

    def __str__(self):
        return f"{self.Path}{self.ObjectName}"

    def get_region(self):
        return self.storage_zone.Region.lower()

    def delete(self):
        return self.call_storage_api(f"/{self.storage_zone.Name}/{self.Path}/{self.ObjectName}", "DELETE")





from json import JSONDecodeError

import requests

from envs import env
from valley.contrib import Schema
# Imports for other modules
from valley.properties import FloatProperty, CharProperty, ListProperty, IntegerProperty, EmailProperty, \
    DateTimeProperty, BooleanProperty, SlugProperty, DictProperty


class BaseBunny(Schema):
    endpoint_url = env('BUNNYCDN_API_ENDPOINT', 'https://bunnycdn.com/api')

    def __init__(self,
                 api_key,
                 endpoint_url=None,
                 **kwargs
                 ):
        self.api_key = api_key
        if endpoint_url:
            self.endpoint_url = endpoint_url
        super().__init__(**kwargs)

    def __repr__(self):
        return '<{class_name}: {uni} >'.format(
            class_name=self.__class__.__name__, uni=self.__str__())

    def get_header(self):
        header = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'accesskey': self.api_key
        }
        return header

    def get_url(self, api_url, endpoint_url):
        if endpoint_url:
            url = f"{endpoint_url}{api_url}"
        else:
            url = f"{self.endpoint_url}{api_url}"
        return url

    def call_api(self, api_url, api_method, header=None, params={}, data={}, json_data={}, endpoint_url=None):
        if not header:
            header = self.get_header()
        r = requests.request(
                method=api_method, url=self.get_url(api_url, endpoint_url), headers=header, params=params, data=data,
                json=json_data)
        try:
            return r.json()
        except JSONDecodeError:
            return r.content


class BaseStorageBunny(BaseBunny):
    storage_endpoint_url = env('BUNNYCDN_STORAGE_API_ENDPOINT', 'storage.bunnycdn.com')

    def get_storage_header(self):
        return {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'AccessKey': self.api_key
        }

    def get_storage_endpoint(self, region):
        return f"https://{region}.{self.storage_endpoint_url}"

    def get_region(self):
        return self.Region.lower()

    def call_storage_api(self, api_url, api_method, header=None, params={}, data={}, json_data={}, files=None,
                         endpoint_url=None):
        if not header:
            header = self.get_storage_header()
        if not endpoint_url:
            endpoint_url = self.get_storage_endpoint(self.get_region())
        if files:
            return requests.put(self.get_url(api_url, endpoint_url), headers=header, files=files)
        return self.call_api(api_url, api_method, header=header, params={}, data=data, json_data=json_data,
                             endpoint_url=endpoint_url)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bunnyhop']

package_data = \
{'': ['*']}

install_requires = \
['envs>=1.3,<2.0', 'requests>=2.23.0,<3.0.0', 'valley>=1.5.5,<2.0.0']

setup_kwargs = {
    'name': 'bunnyhop',
    'version': '0.1.0',
    'description': 'A Python library created make building with BunnyCDN easier',
    'long_description': None,
    'author': 'Brian Jinwright',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from bunnyhop.billing import Billing
from bunnyhop.purge import Purge
from bunnyhop.stats import Stats
from bunnyhop.storage import Storage, StorageZone
from bunnyhop.zone import Zone


class Bunny(object):

    def __init__(self, api_key):
        self.Zone = Zone(api_key)
        self.Purge = Purge(api_key)
        self.Storage = Storage(api_key)
        self.StorageZone = StorageZone(api_key)
        self.Stats = Stats(api_key)
        self.Billing = Billing(api_key)
